@@ -32,6 +32,7 @@ from bokeh.embed import file_html
 from bokeh.resources import INLINE
 from IPython.display import display
 from PIL import Image
+from PIL.Image import Resampling
 from selenium import webdriver
 from selenium.webdriver.chrome.options import Options
 
@@ -53,12 +54,14 @@ class Chart:
 
     """
 
-    def __init__(self,
-                 blank_labels=options.get_option('chart.blank_labels'),
-                 layout='slide_100%',
-                 x_axis_type='linear',
-                 y_axis_type='linear',
-                 second_y_axis=False):
+    def __init__(
+        self,
+        blank_labels=options.get_option("chart.blank_labels"),
+        layout="slide_100%",
+        x_axis_type="linear",
+        y_axis_type="linear",
+        second_y_axis=False,
+    ):
         """Create a chart instance.
 
         Args:
@@ -88,46 +91,41 @@ class Chart:
             plotting methods available.
         """
         # Validate axis type input
-        valid_x_axis_types = [
-            'linear', 'log', 'datetime', 'categorical', 'density'
-        ]
-        valid_y_axis_types = ['linear', 'log', 'categorical', 'density']
-        valid_second_y_axis_types = ['linear', 'log']
+        valid_x_axis_types = ["linear", "log", "datetime", "categorical", "density"]
+        valid_y_axis_types = ["linear", "log", "categorical", "density"]
+        valid_second_y_axis_types = ["linear", "log"]
         if x_axis_type not in valid_x_axis_types:
-            raise ValueError('x_axis_type must be one of {options}'.format(
-                options=valid_x_axis_types))
+            raise ValueError("x_axis_type must be one of {options}".format(options=valid_x_axis_types))
         if y_axis_type not in valid_y_axis_types:
-            raise ValueError('y_axis_type must be one of {options}'.format(
-                options=valid_y_axis_types))
+            raise ValueError("y_axis_type must be one of {options}".format(options=valid_y_axis_types))
 
         self._second_y_axis_type = None
         if second_y_axis:
             self._second_y_axis_type = y_axis_type
             if self._second_y_axis_type not in valid_second_y_axis_types:
                 raise ValueError(
-                    'second_y_axis can only be used when \
-                    y_axis_type is one of {options}'.format(
-                        options=valid_second_y_axis_types))
+                    "second_y_axis can only be used when \
+                    y_axis_type is one of {options}".format(
+                        options=valid_second_y_axis_types
+                    )
+                )
 
         self._x_axis_type, self._y_axis_type = x_axis_type, y_axis_type
 
         self._blank_labels = options._get_value(blank_labels)
         self.style = Style(self, layout)
-        self.figure = self._initialize_figure(self._x_axis_type,
-                                              self._y_axis_type)
-        self.style._apply_settings('chart')
-        self.plot = BasePlot._get_plot_class(self._x_axis_type,
-                                             self._y_axis_type)(self)
+        self.figure = self._initialize_figure(self._x_axis_type, self._y_axis_type)
+        self.style._apply_settings("chart")
+        self.plot = BasePlot._get_plot_class(self._x_axis_type, self._y_axis_type)(self)
         self.callout = Callout(self)
-        self.axes = BaseAxes._get_axis_class(self._x_axis_type,
-                                             self._y_axis_type)(self)
+        self.axes = BaseAxes._get_axis_class(self._x_axis_type, self._y_axis_type)(self)
 
         if self._second_y_axis_type in valid_second_y_axis_types:
             self.second_axis = SecondAxis()
             self.second_axis.axes = SecondYNumericalAxis(self)
-            self.second_axis.plot = BasePlot._get_plot_class(
-                self._x_axis_type, self._second_y_axis_type)(
-                    self, self.second_axis.axes._y_range_name)
+            self.second_axis.plot = BasePlot._get_plot_class(self._x_axis_type, self._second_y_axis_type)(
+                self, self.second_axis.axes._y_range_name
+            )
         self._source = self._add_source_to_figure()
         self._subtitle_glyph = self._add_subtitle_to_figure()
         self.figure.toolbar.logo = None  # Remove bokeh logo from toolbar.
@@ -148,33 +146,35 @@ chartify.Chart(blank_labels={blank_labels},
 layout='{layout}',
 x_axis_type='{x_axis_type}',
 y_axis_type='{y_axis_type}')
-""".format(blank_labels=self._blank_labels,
-           layout=self.style._layout,
-           x_axis_type=self._x_axis_type,
-           y_axis_type=self._y_axis_type)
+""".format(
+            blank_labels=self._blank_labels,
+            layout=self.style._layout,
+            x_axis_type=self._x_axis_type,
+            y_axis_type=self._y_axis_type,
+        )
 
     def _initialize_figure(self, x_axis_type, y_axis_type):
-        x_range, y_range = None, None
-        if x_axis_type == 'categorical':
-            x_range = []
-            x_axis_type = 'auto'
-        if y_axis_type == 'categorical':
-            y_range = []
-            y_axis_type = 'auto'
-        if x_axis_type == 'density':
-            x_axis_type = 'linear'
-        if y_axis_type == 'density':
-            y_axis_type = 'linear'
+        range_args = {}
+        if x_axis_type == "categorical":
+            range_args["x_range"] = []
+            x_axis_type = "auto"
+        if y_axis_type == "categorical":
+            range_args["y_range"] = []
+            y_axis_type = "auto"
+        if x_axis_type == "density":
+            x_axis_type = "linear"
+        if y_axis_type == "density":
+            y_axis_type = "linear"
         figure = bokeh.plotting.figure(
-            x_range=x_range,
-            y_range=y_range,
+            **range_args,
             y_axis_type=y_axis_type,
             x_axis_type=x_axis_type,
-            plot_width=self.style.plot_width,
-            plot_height=self.style.plot_height,
-            tools='save',
+            width=self.style.plot_width,
+            height=self.style.plot_height,
+            tools="save",
             # toolbar_location='right',
-            active_drag=None)
+            active_drag=None
+        )
         return figure
 
     def _add_subtitle_to_figure(self, subtitle_text=None):
@@ -184,16 +184,15 @@ y_axis_type='{y_axis_type}')
                 subtitle_text = ""
             else:
                 subtitle_text = """ch.set_subtitle('Data Description')"""
-        subtitle_settings = self.style._get_settings('subtitle')
+        subtitle_settings = self.style._get_settings("subtitle")
         _subtitle_glyph = bokeh.models.Title(
             text=subtitle_text,
-            align=subtitle_settings['subtitle_align'],
-            text_color=subtitle_settings['subtitle_text_color'],
-            text_font_size=subtitle_settings['subtitle_text_size'],
-            text_font=subtitle_settings['subtitle_text_font'],
-            )
-        self.figure.add_layout(_subtitle_glyph,
-                               subtitle_settings['subtitle_location'])
+            align=subtitle_settings["subtitle_align"],
+            text_color=subtitle_settings["subtitle_text_color"],
+            text_font_size=subtitle_settings["subtitle_text_size"],
+            text_font=subtitle_settings["subtitle_text_font"],
+        )
+        self.figure.add_layout(_subtitle_glyph, subtitle_settings["subtitle_location"])
         return _subtitle_glyph
 
     def _add_source_to_figure(self):
@@ -201,20 +200,21 @@ y_axis_type='{y_axis_type}')
         source_text = """ch.set_source_label('Source')"""
         if self._blank_labels:
             source_text = ""
-        source_text_color = '#898989'
-        source_font_size = '10px'
+        source_text_color = "#898989"
+        source_font_size = "10px"
         _source = bokeh.models.Label(
-            x=self.style.plot_width * .9,
+            x=self.style.plot_width * 0.9,
             y=0,
-            x_units='screen',
-            y_units='screen',
-            level='overlay',
+            x_units="screen",
+            y_units="screen",
+            level="overlay",
             text=source_text,
             text_color=source_text_color,
             text_font_size=source_font_size,
-            text_align='right',
-            name='subtitle')
-        self.figure.add_layout(_source, 'below')
+            text_align="right",
+            name="subtitle",
+        )
+        self.figure.add_layout(_source, "below")
         return _source
 
     @property
@@ -224,9 +224,7 @@ y_axis_type='{y_axis_type}')
         Note:
             The format will depend on the types of plots that have been added.
         """
-        datasources = self.figure.select({
-            'type': bokeh.models.ColumnDataSource
-        })
+        datasources = self.figure.select({"type": bokeh.models.ColumnDataSource})
         # Extract the data attribute from the ColumnDataSource object
         # and place in a list.
         datasources_list = list(map(lambda x: x.data, datasources))
@@ -291,7 +289,7 @@ y_axis_type='{y_axis_type}')
         """str: Legend location."""
         return self.figure.legend[0].location
 
-    def set_legend_location(self, location, orientation='horizontal'):
+    def set_legend_location(self, location, orientation="horizontal"):
         """Set the legend location.
 
         Args:
@@ -316,23 +314,24 @@ y_axis_type='{y_axis_type}')
                     """
                     Legend location will not apply.
                     Set the legend after plotting data.
-                    """, UserWarning)
+                    """,
+                    UserWarning,
+                )
                 return self
             new_legend = self.figure.legend[0]
             new_legend.orientation = orientation
             self.figure.add_layout(new_legend, layout_location)
 
-        if location == 'outside_top':
-            add_outside_legend('top_left', 'above')
+        if location == "outside_top":
+            add_outside_legend("top_left", "above")
             # Re-render the subtitle so that it appears over the legend.
             subtitle_index = self.figure.renderers.index(self._subtitle_glyph)
             self.figure.renderers.pop(subtitle_index)
-            self._subtitle_glyph = self._add_subtitle_to_figure(
-                self._subtitle_glyph.text)
-        elif location == 'outside_bottom':
-            add_outside_legend('bottom_center', 'below')
-        elif location == 'outside_right':
-            add_outside_legend('top_left', 'right')
+            self._subtitle_glyph = self._add_subtitle_to_figure(self._subtitle_glyph.text)
+        elif location == "outside_bottom":
+            add_outside_legend("bottom_center", "below")
+        elif location == "outside_right":
+            add_outside_legend("top_left", "right")
         elif location is None:
             self.figure.legend.visible = False
         else:
@@ -342,12 +341,11 @@ y_axis_type='{y_axis_type}')
         vertical = self.axes._vertical
         # Reverse the legend order
         if self._reverse_vertical_legend:
-            if orientation == 'vertical' and vertical:
-                self.figure.legend[0].items = list(
-                    reversed(self.figure.legend[0].items))
+            if orientation == "vertical" and vertical:
+                self.figure.legend[0].items = list(reversed(self.figure.legend[0].items))
         return self
 
-    def show(self, format='html'):
+    def show(self, format="html"):
         """Show the chart.
 
         Args:
@@ -365,20 +363,20 @@ y_axis_type='{y_axis_type}')
                     Recommended when the plot is in a finished state.
 
                 - 'svg': Output as SVG.
-                """
+        """
         self._set_toolbar_for_format(format)
 
-        if format == 'html':
+        if format == "html":
             return bokeh.io.show(self.figure)
-        elif format == 'png':
+        elif format == "png":
             image = self._figure_to_png()
             # Need to re-enable this when logos are added back.
             # image = self.logo._add_logo_to_image(image)
             return display(image)
-        elif format == 'svg':
+        elif format == "svg":
             return self._show_svg()
 
-    def save(self, filename, format='html'):
+    def save(self, filename, format="html"):
         """Save the chart.
 
         Args:
@@ -400,48 +398,49 @@ y_axis_type='{y_axis_type}')
         """
         self._set_toolbar_for_format(format)
 
-        if format == 'html':
+        if format == "html":
             bokeh.io.saving.save(
                 self.figure,
                 filename=filename,
                 resources=INLINE,
-                title='Chartify chart.')
-        elif format == 'png':
+                title="Chartify chart.",
+            )
+        elif format == "png":
             image = self._figure_to_png()
             # Need to re-enable this when logos are added back.
             # image = self.logo._add_logo_to_image(image)
             image.save(filename)
-        elif format == 'svg':
+        elif format == "svg":
             image = self._figure_to_svg()
             self._save_svg(image, filename)
 
-        print('Saved to {filename}'.format(filename=filename))
+        print("Saved to {filename}".format(filename=filename))
 
         return self
 
     def _set_toolbar_for_format(self, format):
-        if format == 'html':
-            self.figure.toolbar_location = 'right'
-        elif format in ('png', 'svg'):
+        if format == "html":
+            self.figure.toolbar_location = "right"
+        elif format in ("png", "svg"):
             self.figure.toolbar_location = None
         elif format is None:  # If format is None the chart won't be shown.
             pass
         else:
-            raise ValueError(
-                """Invalid format. Valid options are 'html', 'png' or 'svg'.""")
+            raise ValueError("""Invalid format. Valid options are 'html', 'png' or 'svg'.""")
 
     def _initialize_webdriver(self):
         """Initialize headless chrome browser"""
         options = Options()
-        options.add_argument("window-size={width},{height}".format(
-            width=self.style.plot_width, height=self.style.plot_height))
+        options.add_argument(
+            "window-size={width},{height}".format(width=self.style.plot_width, height=self.style.plot_height)
+        )
         options.add_argument("start-maximized")
         options.add_argument("disable-infobars")
         options.add_argument("disable-gpu")
-        options.add_argument('no-sandbox')  # Required for use in docker.
+        options.add_argument("no-sandbox")  # Required for use in docker.
         options.add_argument("--disable-extensions")
-        options.add_argument('--headless')
-        options.add_argument('--hide-scrollbars')
+        options.add_argument("--headless")
+        options.add_argument("--hide-scrollbars")
         driver = webdriver.Chrome(options=options)
         return driver
 
@@ -453,8 +452,7 @@ y_axis_type='{y_axis_type}')
         driver = self._initialize_webdriver()
         # Save figure as HTML
         html = file_html(self.figure, resources=INLINE, title="")
-        fp = tempfile.NamedTemporaryFile(
-            'w', prefix='chartify', suffix='.html', encoding='utf-8')
+        fp = tempfile.NamedTemporaryFile("w", prefix="chartify", suffix=".html", encoding="utf-8")
         fp.write(html)
         fp.flush()
         # Open html file in the browser.
@@ -467,18 +465,20 @@ y_axis_type='{y_axis_type}')
         image = Image.open(BytesIO(png))
         target_dimensions = (self.style.plot_width, self.style.plot_height)
         if image.size != target_dimensions:
-            image = image.resize(target_dimensions, resample=Image.LANCZOS)
+            image = image.resize(target_dimensions, resample=Resampling.LANCZOS)
         return image
 
     def _set_svg_backend_decorator(f):
         """Sets the chart backend to svg and resets
         after the function has run."""
+
         @wraps(f)
         def wrapper(self, *args, **kwargs):
             old_backend = self.figure.output_backend
-            self.figure.output_backend = 'svg'
+            self.figure.output_backend = "svg"
             return f(self, *args, **kwargs)
             self.figure.output_backend = old_backend
+
         return wrapper
 
     @_set_svg_backend_decorator
@@ -495,8 +495,7 @@ y_axis_type='{y_axis_type}')
         driver = self._initialize_webdriver()
         html = file_html(self.figure, resources=INLINE, title="")
 
-        fp = tempfile.NamedTemporaryFile(
-            'w', prefix='chartify', suffix='.html', encoding='utf-8')
+        fp = tempfile.NamedTemporaryFile("w", prefix="chartify", suffix=".html", encoding="utf-8")
         fp.write(html)
         fp.flush()
         driver.get("file:///" + fp.name)
@@ -514,14 +513,12 @@ y_axis_type='{y_axis_type}')
 
 
 class Logo:
-
     def __init__(self, chart):
         self._chart = chart
         self._logo_image = None
-        self._path = options.get_option('config.logos_path')
+        self._path = options.get_option("config.logos_path")
         self._logo_file_mapping = {}
-        self._logo_file_mapping = OrderedDict(
-            sorted(list(self._logo_file_mapping.items()), key=lambda t: t[0]))
+        self._logo_file_mapping = OrderedDict(sorted(list(self._logo_file_mapping.items()), key=lambda t: t[0]))
 
     def _add_logo_to_image(self, image):
         """If the logo is set then add it to the chart image."""
@@ -536,20 +533,19 @@ class Logo:
         return image
 
     def _resize_logo(self, logo_image):
-
         logo_width, logo_height = logo_image.size
 
         # TODO smart scaling of logos
-        target_height = int(self._chart.style.plot_height * .1)
+        target_height = int(self._chart.style.plot_height * 0.1)
 
         if logo_width == logo_height:
-            logo_image = logo_image.resize(
-                (target_height, target_height), resample=Image.LANCZOS)
+            logo_image = logo_image.resize((target_height, target_height), resample=Image.LANCZOS)
         else:
             logo_width_to_height = logo_width * 1.0 / logo_height
             logo_image = logo_image.resize(
                 (int(logo_width_to_height * target_height), target_height),
-                resample=Image.LANCZOS)
+                resample=Image.LANCZOS,
+            )
         return logo_image
 
     def show_logo_options(self):
@@ -569,8 +565,10 @@ class Logo:
             filename = self._logo_file_mapping[logo]
         except KeyError:
             raise KeyError(
-                'Must supply a valid logo name: {valid_options}'.format(
-                    valid_options=list(self._logo_file_mapping.keys())))
+                "Must supply a valid logo name: {valid_options}".format(
+                    valid_options=list(self._logo_file_mapping.keys())
+                )
+            )
 
         logo_image = Image.open(self._path + filename)
 
